require 'sketchup' # First we pull in the standard API hooks.
require 'ShaDe//lib//utils.rb'
require 'ShaDe//lib//geometry.rb'
require 'ShaDe//lib//data-structures.rb'
require 'ShaDe//lib//main-structures.rb'
require 'ShaDe//lib//constraints.rb'
require 'ShaDe//lib//goals.rb'
require 'ShaDe//lib//interfaces//custom.rb'

#Author:: Manuela Ruiz  (mailto:mruiz@lcc.uma.es)
#This class represents the observer for a rule shape group. There is one observer of this kind for every present rule shape.
class RuleShapeObserver < Sketchup::EntitiesObserver
	
	#Initializing
	def initialize(shape)
		@shape = shape
	end
	
	#Method that triggers when something is modified inside a rule shape group, in order to update the internal representation of the affected rule shape
	def onElementModified(entities, entity)
		
		if !Shade.project.modifying
			if !Shade.project.erasing
				Shade.project.modifying = true
				execution = Shade.project.execution
				layer_name = Sketchup.active_model.active_layer.name
				@shape.refresh_from_entities(entities, @shape.transformation(layer_name), layer_name)
				execution.grammar.saved = false
				Shade.project.modifying = false
			end
		end
	end

	#Method that triggers when a rule shape group is erased, in order to update the internal representation of the affected rule shape
	def onEraseEntities(entities)
		
		if !Shade.project.modifying
			Shade.project.modifying = true
			layer_name = Sketchup.active_model.active_layer.name
			@shape.p[layer_name] = LinearLinkedList.new
			@shape.s[layer_name] = LinearLinkedList.new
			execution = Shade.project.execution
			execution.grammar.saved = false
			Shade.project.modifying = false
		end
	end

	#Method that triggers when some element is added into a rule shape group, in order to update the internal representation of the affected rule shape
	def onElementAdded(entities, entity)
		
		if !Shade.project.modifying
			
			Shade.project.modifying = true
			execution = Shade.project.execution
			layer_name = Sketchup.active_model.active_layer.name
			@shape.refresh_from_entities(entities, @shape.transformation(layer_name), layer_name)
			execution.grammar.saved = false
			Shade.project.modifying = false
		end
	end
	
	#Method that triggers when some element is removed from a rule shape group, in order to update the internal representation of the affected rule shape
	def onElementRemoved(entities, entity_id)
		
		if !Shade.project.modifying
			Shade.project.modifying = true
			execution = Shade.project.execution
			layer_name = Sketchup.active_model.active_layer.name
			@shape.refresh_from_entities(entities, @shape.transformation(layer_name), layer_name)
			execution.grammar.saved = false
			Shade.project.modifying = false
		end
	end
end

#Author:: Manuela Ruiz  (mailto:mruiz@lcc.uma.es)
#This class represents the observer for all the rule shape groups. There is only one observer of this kind.
class RuleGroupsObserver < Sketchup::EntitiesObserver
	
	attr_accessor :observed_id_list
	
	#Initializing the observer
	def initialize()
		#List of entity ids that cannot be modified
		#These entities are the main groups of the rule shapes
		@observed_id_list = Array.new
		@undoing = false
		super
	end
	
	#Method that triggers when some rule shape group is modified (transformed) as a whole, in order to update the internal representation of the affected rule shape
	def onElementModified(entities, entity)
		
		if @observed_id_list.include? entity.entityID
			
			if !Shade.project.modifying
				Shade.project.modifying = true
				execution = Shade.project.execution
				shape = execution.grammar.search_shape_by_id(entity.entityID)

				if shape
					#refresh the transformation										
					layout_t = shape.layout_transformation
					layout_t_i = layout_t.inverse
					shape_t = layout_t_i * entity.transformation
					
					layer_name = Sketchup.active_model.active_layer.name
					shape.shape_transformation[layer_name] = shape_t
					
					shape.refresh_from_entities(entity.entities, Geom::Transformation.new, layer_name)
					
					execution.grammar.saved = false
				end
				Shade.project.modifying = false
			end
		end
	end
	
end



#Author:: Manuela Ruiz  (mailto:mruiz@lcc.uma.es)
# Observer attached to  Sketchup. Triggers when SU is closed
class CloseObserver < Sketchup::AppObserver	
	#Asks to save the project and removes the observers
	def onQuit
		ShadeUtils.ask_to_save_project()
	end
	
end

#Author:: Manuela Ruiz  (mailto:mruiz@lcc.uma.es)
#Observer attached to Sketchup.active_model, triggers when the user makes an UNDO
class UndoObserver < Sketchup::ModelObserver
	
	#Reexecutes the grammar and shows a message, saying that it is not allowed to undo
	def onTransactionUndo(model)
		Sketchup.active_model.start_operation("Undo Done")
		#Shade.project.need_refresh = true
		UI.messagebox("We are sorry, you cannot use the undo command.")
		Sketchup.active_model.commit_operation
	end
end
#Author:: Manuela Ruiz  (mailto:mruiz@lcc.uma.es)
#Tool for painting a label
class AddLabelTool

	#shape:: the shape to which the label is going to be added
	#value:: the value of the label
	#
	#Initializing the tool
	def initialize(shape, value)
		@id_cursor = nil
		@shape = shape
		@value = value
	end
	
	#Activates the cursor
	def activate
		file_name = Sketchup.find_support_file "add_label.png", "Plugins/ShaDe/#{Constants::ABS_ICONS_DIR}"
		@id_cursor = UI.create_cursor file_name, 18, 18
	end
	
	#Method that triggers when the cursor is chosen
	def onSetCursor
		UI.set_cursor  @id_cursor
	end
	
	#Method that triggers when the left button of the mouse is pressed
	def onLButtonDown(flags, x, y, view)

		inputpoint = view.inputpoint x,y
		point = inputpoint.position
		
		#Obtain point relative to shape
		t = @shape.layout_transformation.inverse
		pt_label = t * point
		layer_name = Sketchup.active_model.active_layer.name
		@shape.add_label(pt_label, @value, layer_name)
		
		@shape.changed = true
		
		Shade.project.refresh
		Shade.project.execution.reset
	end
end

#Saves the project, the current shape and the execution history
def save_temp_files()
	project = Shade.project
	
	#STEP 1: Save project
	project_path = "#{File.dirname(__FILE__)}/#{Constants::TEMP_DIR}/project.txt"
	project_path.gsub!("/", "\\")
	project.save(project_path, true)

	#STEP 2: Save current shape 
	current_shape_path = "#{File.dirname(__FILE__)}/#{Constants::TEMP_DIR}/current_shape.txt"
	current_shape_path.gsub!("/", "\\")
	project.execution.current_shape.save(current_shape_path)

	#STEP 2: Save execution history
	history_directory = "#{File.dirname(__FILE__)}/#{Constants::TEMP_DIR}"
	history_directory.gsub!("/", "\\")
	project.execution.save_execution_history(history_directory)
end

#Loads the temp files
def load_temp_files()
	execution = Shade.project.execution

	#STEP 1: Load execution history
	history_directory = "#{File.dirname(__FILE__)}/#{Constants::TEMP_DIR}"
	history_directory.gsub!("/", "\\")
	execution.load_execution_history(history_directory)
	
	#STEP 1: Load current shape
	current_shape_path = "#{File.dirname(__FILE__)}/#{Constants::TEMP_DIR}/current_shape.txt"
	current_shape_path.gsub!("/", "\\")
	execution.current_shape.load(current_shape_path)
	execution.current_shape.refresh
	execution.current_shape.create_pi
end

#Deletes all the files inside temp folder
def delete_temp_files()
	dir = Dir.new("#{File.dirname(__FILE__)}/#{Constants::TEMP_DIR}")
	dir.each { |file_name|
		if file_name == '.' or file_name == '..' then next
		else File.delete("#{File.dirname(__FILE__)}/#{Constants::TEMP_DIR}/#{file_name}")
		end
	}
end

#Method that creates the toolbar for saving and loading projects, grammars and shapes, as well as editing rules
def create_static_toolbar()
	
	toolbar = UI.toolbar Constants::STATIC_TOOLBAR_NAME
	
	# Create robot command
	create_robot_command = UI::Command.new("create_script"){
		Sketchup.active_model.close_active
		# Find and show our html file
		html_path = Sketchup.find_support_file "createRobot.html" , Constants::HTML_DIR
		robot_dialog = create_robot_dialog
		robot_dialog.set_file(html_path)
		robot_dialog.show()
	}
	create_robot_command.tooltip = "Create script"
	create_robot_command.small_icon = File.join(Constants::ICONS_DIR, "create_robot.PNG")
	create_robot_command.large_icon = File.join(Constants::ICONS_DIR, "create_robot.PNG")
	toolbar.add_item create_robot_command
	
	# Load robot command
	load_robot_command = UI::Command.new("load_script"){
		Sketchup.active_model.close_active
		chosen_robot_path = UI.openpanel("Open script", "" ,"*.txt")
		if chosen_robot_path			
			robot = Robot.new
			robot.load(chosen_robot_path)
			Shade.robot = robot
		end
	}
	load_robot_command.tooltip = "Load script"
	load_robot_command.small_icon = File.join(Constants::ICONS_DIR, "load_robot.PNG")
	load_robot_command.large_icon = File.join(Constants::ICONS_DIR, "load_robot.PNG")
	toolbar.add_item load_robot_command
	
	toolbar.add_separator()

	#New Project command
	newp_cmd = UI::Command.new("new_project"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active
		execution.reset

		#Before creating a new project, we save the current one if necessary
		if !project.saved
			input = UI.messagebox("Save current project?", MB_YESNOCANCEL)
			if input == 6
				if project.path
					project.save(project.path, true)
				else
					saved = false
					while !saved
						path_to_save_to = UI.savepanel "Save Project", "", "project.prj"
						if path_to_save_to
							if ShadeUtils.get_extension(path_to_save_to) == "prj"
								begin
									project.save(path_to_save_to, true)
									saved = true
								rescue
									UI.messagebox("The path you have chosen is not valid (maybe it has some special character?)")
								end
							else
								UI.messagebox("Please save the project as a .prj file")
							end
						else
							saved = true
						end
					 end
				end
			end
		end    
		ShadeUtils.create_default_new_project()
	}
	newp_cmd.tooltip = "New project"
	newp_cmd.small_icon = File.join(Constants::ICONS_DIR, "new_project.PNG")
	newp_cmd.large_icon = File.join(Constants::ICONS_DIR, "new_project.PNG")
	toolbar.add_item newp_cmd
	
	#Open Project command
	openp_cmd = UI::Command.new("open_project"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active
		#execution.reset
    
		if !project.saved
			input = UI.messagebox("Save current project?", MB_YESNOCANCEL)
			if input == 6
				if project.path
					project.save(project.path, true)
				else
					saved = false
					while !saved
						path_to_save_to = UI.savepanel "Save Project", "", "project.prj"
						if path_to_save_to
							if ShadeUtils.get_extension(path_to_save_to) == "prj"
								begin
									project.save(path_to_save_to, true)
									saved = true
								rescue
									UI.messagebox("The path you have chosen is not valid (maybe it has some special character?)")
								end
							else
								UI.messagebox("Please save the project as a .prj file")
							end
						else
							saved = true
						end
					end
				end
			end
		end

		opened = false
		while !opened
			chosen_project_path = UI.openpanel("Open project", "" ,"*.prj")
			if chosen_project_path	
				if ShadeUtils.get_extension(chosen_project_path) == "prj"				
					begin
						project.load(chosen_project_path)
						project.saved = true
						project.refresh
						opened = true
					rescue
						UI.messagebox("The path you have chosen is not valid (maybe it has some special character?)")
					rescue LoadError => e
						UI.messagebox(e.message)
					end
				else
					UI.messagebox("Please choose a .prj file")
				end
			else
				opened = true
			end
		end
  
	}
	openp_cmd.tooltip = "Open project"
	openp_cmd.small_icon = File.join(Constants::ICONS_DIR, "open_project.PNG")
	openp_cmd.large_icon = File.join(Constants::ICONS_DIR, "open_project.PNG")
	toolbar.add_item openp_cmd
	
	#Save Project command
	savep_cmd = UI::Command.new("save_project"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active
    
		if project.path
			project.save(project.path, true)
		else
			saved = false
			while !saved
				path_to_save_to = UI.savepanel "Save Project", "", "project.prj"
				if path_to_save_to
					if ShadeUtils.get_extension(path_to_save_to) == "prj"
						begin
							project.save(path_to_save_to, true)
							saved = true
						rescue
							UI.messagebox("The path you have chosen is not valid (maybe it has some special character?)")
						end
					else
						UI.messagebox("Please save the project as a .prj file")
					end
				else
					saved = true
				end
			end
		end
	}
	savep_cmd.tooltip = "Save project"
	savep_cmd.small_icon = File.join(Constants::ICONS_DIR, "save_project.PNG")
	savep_cmd.large_icon = File.join(Constants::ICONS_DIR, "save_project.PNG")
	toolbar.add_item savep_cmd
	
	#Save Project as command
	savepas_cmd = UI::Command.new("save_project_as"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active
		
		saved = false
		while !saved
			path_to_save_to = UI.savepanel "Save Project", "", "project.prj"
			if path_to_save_to
				if ShadeUtils.get_extension(path_to_save_to) == "prj"
					begin
						project.save(path_to_save_to, true)
						saved = true
					rescue
						UI.messagebox("The path you have chosen is not valid (maybe it has some special character?)")
					end
				else
					UI.messagebox("Please save the project as a .prj file")
				end
			else
				saved = true
			end
		end

	}
	savepas_cmd.tooltip = "Save project as"
	savepas_cmd.small_icon = File.join(Constants::ICONS_DIR, "save_project_as.PNG")
	savepas_cmd.large_icon = File.join(Constants::ICONS_DIR, "save_project_as.PNG")
	toolbar.add_item savepas_cmd
	
	toolbar.add_separator
	
	#New Grammar command
	newg_cmd = UI::Command.new("new_grammar"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active
		execution.reset

		if !execution.grammar.saved
			input = UI.messagebox("Save current grammar?", MB_YESNOCANCEL)
			if input == 6
				saved = false
				while !saved
					path_to_save_to = UI.savepanel "Save Grammar", "", "grammar.gr2"
					if path_to_save_to
						if ShadeUtils.get_extension(path_to_save_to) == "gr2"
							begin
								execution.grammar.save(path_to_save_to, true)
								saved = true
							rescue
								UI.messagebox("The path you have chosen is not valid (maybe it has some special character?)")
							end
						else
							UI.messagebox("Please save the grammar as a .gr2 file")
						end
					else
						saved = true
					end
				end
			end
		end
		ShadeUtils.create_default_new_grammar()
		project.saved = false
		project.refresh
	}
	newg_cmd.tooltip = "New grammar"
	newg_cmd.small_icon = File.join(Constants::ICONS_DIR, "new_grammar.PNG")
	newg_cmd.large_icon = File.join(Constants::ICONS_DIR, "new_grammar.PNG")
	toolbar.add_item newg_cmd
	
	#Open Grammar command
	openg_cmd = UI::Command.new("open_grammar"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active
		execution.reset

		if !execution.grammar.saved
			input = UI.messagebox("Save current grammar?", MB_YESNOCANCEL)
			if input == 6
				saved = false
				while !saved
					path_to_save_to = UI.savepanel "Save Grammar", "", "grammar.gr2"
					if path_to_save_to
						if ShadeUtils.get_extension(path_to_save_to) == "gr2"
							begin
								execution.grammar.save(path_to_save_to, true)
								saved = true
							rescue
								UI.messagebox("The path you have chosen is not valid (maybe it has some special character?)")
							end
						else
							UI.messagebox("Please save the grammar as a .gr2 file")
						end
					else
						saved = true
					end
				end
			end
		end

		opened = false
		while !opened
			chosen_grammar_path = UI.openpanel("Open grammar", "" ,"*.gr2")
			if chosen_grammar_path
				if ShadeUtils.get_extension(chosen_grammar_path) == "gr2"
					begin
						execution.grammar.load(chosen_grammar_path)
						opened = true
						project.saved = false
						project.refresh
					rescue LoadError => e
						UI.messagebox(e.message)
					rescue
						UI.messagebox("The path you have chosen is not valid (maybe it has some special character?)")
					end
				else
					UI.messagebox("Please choose a .gr2 file")
				end
			else
				opened = true
			end
		end
	}
	openg_cmd.tooltip = "Open grammar"
	openg_cmd.small_icon = File.join(Constants::ICONS_DIR, "open_grammar.PNG")
	openg_cmd.large_icon = File.join(Constants::ICONS_DIR, "open_grammar.PNG")
	toolbar.add_item openg_cmd
	
	#Save Grammar As command
	savegas_cmd = UI::Command.new("save_grammar_as"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active
		saved = false
		while !saved
			path_to_save_to = UI.savepanel "Save Grammar", "", "grammar.gr2"
			if path_to_save_to
				if ShadeUtils.get_extension(path_to_save_to) == "gr2"
					begin
						execution.grammar.save(path_to_save_to, true)
						saved = true
					rescue
						UI.messagebox("The path you have chosen is not valid (maybe it has some special character?)")
					end
				else
					UI.messagebox("Please save the grammar as a .gr2 file")
				end
			else
				saved = true
			end
		end
	}
	savegas_cmd.tooltip = "Save grammar as"
	savegas_cmd.small_icon = File.join(Constants::ICONS_DIR, "save_grammar_as.PNG")
	savegas_cmd.large_icon = File.join(Constants::ICONS_DIR, "save_grammar_as.PNG")
	toolbar.add_item savegas_cmd
	
	toolbar.add_separator()
	
	# Load shape command
	load_shape_cmd = UI::Command.new("load_shape"){
		Sketchup.active_model.close_active
		prompts = ["Rule ID ", "Rule part "]
		default = ["1", "Left"]
		rule_list = ShadeUtils.create_rule_list()
		list = [rule_list, "Left|Right"]
		input = UI.inputbox prompts, default, list, "Choose shape:"
		if input
			rule_idx = input[0].to_i - 1
			rule = Shade.project.execution.grammar.rules[rule_idx]
			if input[1] == "Left"
				shape = rule.left
			else
				shape = rule.right
			end
			opened = false
			while !opened
				load_shape_path = UI.openpanel "Load Shape", "", "*.txt"
				if load_shape_path
					if ShadeUtils.get_extension(load_shape_path) == "txt"
						begin
							shape.load(load_shape_path)
							opened = true
							if ((rule_idx == 0) && (input[1] == "Left") && (!Shade.project.execution.file_axiom))
								# Add new axiom			
								new_axiom = LabelledShape.new(Array.new, Array.new)
								Sketchup.active_model.layers.each { |layer|
									new_axiom.p = shape.p.clone
									new_axiom.s = shape.s.clone
								}
								Shade.project.execution.grammar.axiom = new_axiom
								Shade.project.execution.reset
							end
						rescue LoadError => e
							UI.messagebox(e.message)
						rescue
							UI.messagebox("The path you have chosen is not valid (maybe it has some special character?)")
						end
					else
						UI.messagebox("Please choose a .txt file")
					end
				else
					opened = true
				end
			end
		end
	}
	load_shape_cmd.tooltip = "Load shape"
	load_shape_cmd.small_icon = File.join(Constants::ICONS_DIR, "load_shape.PNG")
	load_shape_cmd.large_icon = File.join(Constants::ICONS_DIR, "load_shape.PNG")
	toolbar.add_item load_shape_cmd
	
	# Save shape command
	save_shape_cmd = UI::Command.new("save_shape"){
		Sketchup.active_model.close_active
		prompts = ["Rule ID ", "Rule part "]
		default = ["1", "Left"]
		rule_list = ShadeUtils.create_rule_list()
		list = [rule_list, "Left|Right"]
		input = UI.inputbox prompts, default, list, "Choose shape:"
		if input
			rule_idx = input[0].to_i - 1
			rule = Shade.project.execution.grammar.rules[rule_idx]
			if input[1] == "Left"
				shape = rule.left
			else
				shape = rule.right
			end
			saved = false
			while !saved
				save_shape_path = UI.savepanel "Save Shape", "", "shape.txt"
				if save_shape_path
					if ShadeUtils.get_extension(save_shape_path) == "txt"
						begin
							shape.save(save_shape_path)
							saved = true
						rescue
							UI.messagebox("The path you have chosen is not valid (maybe it has some special character?)")
						end
					else
						UI.messagebox("Please save the shape as a .txt file")
					end
				else
					saved = true
				end
			end
		end
	}
	save_shape_cmd.tooltip = "Save shape"
	save_shape_cmd.small_icon = File.join(Constants::ICONS_DIR, "save_shape.PNG")
	save_shape_cmd.large_icon = File.join(Constants::ICONS_DIR, "save_shape.PNG")
	toolbar.add_item save_shape_cmd
	
	toolbar.add_separator()
	
	# Load Axiom command
	load_axiom_cmd = UI::Command.new("load_axiom"){
		Sketchup.active_model.close_active
    
		opened = false
		while !opened
			chosen_axiom_path = UI.openpanel("Load axiom", "" ,"*.txt")
			if chosen_axiom_path
				if ShadeUtils.get_extension(chosen_axiom_path) == "txt"
					begin
						new_axiom = LabelledShape.new(Array.new, Array.new)
						new_axiom.load(chosen_axiom_path)
						Shade.project.execution.file_axiom = true
						Shade.project.execution.grammar.axiom = new_axiom
						Shade.project.execution.reset
						opened = true
					rescue LoadError => e
						UI.messagebox(e.message)
					rescue
						UI.messagebox("The path you have chosen is not valid (maybe it has some special character?)")
					end
				else
					UI.messagebox("Please choose a .txt file")
				end
			else
				opened = true
			end
		end
	}
	load_axiom_cmd.tooltip = "Load Axiom"
	load_axiom_cmd.small_icon = File.join(Constants::ICONS_DIR, "load_axiom.PNG")
	load_axiom_cmd.large_icon = File.join(Constants::ICONS_DIR, "load_axiom.PNG")
	toolbar.add_item load_axiom_cmd
	
	# Save Design command
	save_design_cmd = UI::Command.new("save_design"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active
		
		saved = false
		while !saved
			path_to_save_to = UI.savepanel "Save Design", "", "design.txt"
			if (path_to_save_to)
				if ShadeUtils.get_extension(path_to_save_to) == "txt"
					begin
						execution.current_shape.save(path_to_save_to)
						saved = true
					rescue
						UI.messagebox("The path you have chosen is not valid (maybe it has some special character?)")
					end
				else
					UI.messagebox("Please save the design as a .txt file")
				end
			else
				saved = true
			end
		end
	}
	save_design_cmd.tooltip = "Save Design"
	save_design_cmd.small_icon = File.join(Constants::ICONS_DIR, "save_design.PNG")
	save_design_cmd.large_icon = File.join(Constants::ICONS_DIR, "save_design.PNG")
	toolbar.add_item save_design_cmd
	
	toolbar.add_separator()

	
	# About command
	about_cmd = UI::Command.new("about"){
		UI.messagebox("ShaDe v#{Constants::VERSION}\nAll rights reserved\nFor research use only \n(c) Manuela Ruiz Montiel and Universidad de Malaga")
	}
	about_cmd.tooltip = "About"
	about_cmd.small_icon = File.join(Constants::ICONS_DIR, "about.PNG")
	about_cmd.large_icon = File.join(Constants::ICONS_DIR, "about.PNG") 
	toolbar.add_item about_cmd
	
	toolbar.show
end


def create_edition_toolbar()
	
	toolbar = UI.toolbar Constants::EDITION_TOOLBAR_NAME
	#Add Rule command
	add_rule_cmd = UI::Command.new("add_rule"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active

		size = execution.grammar.rules.size
		
		#Create the default shapes
		left = ShadeUtils.create_default_left_shape
		right = ShadeUtils.create_default_right_shape
		
		#Obtain new rule id
		last_id = execution.grammar.rules[size-1].rule_id
		
		#Create new rule
		new_rule = ShadeUtils.paint_rule(last_id+1, left, right)
		execution.grammar.add_rule(new_rule)
		project.refresh
	}
	add_rule_cmd.tooltip = "Add rule"
	add_rule_cmd.small_icon = File.join(Constants::ICONS_DIR, "add_rule.PNG")
	add_rule_cmd.large_icon = File.join(Constants::ICONS_DIR, "add_rule.PNG")
	toolbar.add_item add_rule_cmd
	
	#Delete Rule command
	delete_rule_cmd = UI::Command.new("delete_rule"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active

		if execution.grammar.rules.size > 1
			prompts = ["Rule to delete: "]
			default = ["1"]
			rule_list = ShadeUtils.create_rule_list()
			list = [rule_list]
			input = UI.inputbox prompts, default, list, "Delete Rule"
			if input
				chosen_rule_idx = input[0].to_i-1
				
				if ((chosen_rule_idx == 0) && (!execution.file_axiom))
					previous_rule = execution.grammar.rules[1]
					# Add new axiom			
					new_axiom = LabelledShape.new(Array.new, Array.new)
					Sketchup.active_model.layers.each { |layer|
						new_axiom.p[layer.name] = previous_rule.left.p[layer.name].clone
						new_axiom.s[layer.name] = previous_rule.left.s[layer.name].clone
					}
					execution.grammar.axiom = new_axiom
					
					execution.reset
				end
				execution.grammar.remove_rule(chosen_rule_idx)
				# Transform the position of the rules from chosen_rule_idx
				idx = chosen_rule_idx
  
				while idx < execution.grammar.rules.size
					
					current_rule = execution.grammar.rules[idx]
					
					current_rule.left.layout_transformation = current_rule.left.layout_transformation * Constants::DESCEND_T.inverse
					current_rule.right.layout_transformation = current_rule.right.layout_transformation * Constants::DESCEND_T.inverse
					
					current_rule.left.changed = true
					current_rule.right.changed = true
			 
					Sketchup.active_model.layers.each { |layer|
						current_rule.arrow_group[layer.name].locked = false
						current_rule.arrow_group[layer.name].transform! Constants::DESCEND_T.inverse
						current_rule.arrow_group[layer.name].locked = true
						
						current_rule.line_group[layer.name].locked = false
						current_rule.line_group[layer.name].transform! Constants::DESCEND_T.inverse
						current_rule.line_group[layer.name].locked = true
						
						current_rule.group_origin_left[layer.name].locked = false
						current_rule.group_origin_left[layer.name].transform! Constants::DESCEND_T.inverse
						current_rule.group_origin_left[layer.name].locked = true
						
						current_rule.group_origin_right[layer.name].locked = false
						current_rule.group_origin_right[layer.name].transform! Constants::DESCEND_T.inverse
						current_rule.group_origin_right[layer.name].locked = true
					}
    
					idx += 1
				end
				execution.reset
				project.refresh
			end
		else
			UI.messagebox("You cannot delete all rules")
		end
	}
	delete_rule_cmd.tooltip = "Delete rule"
	delete_rule_cmd.small_icon = File.join(Constants::ICONS_DIR, "delete_rule.PNG")
	delete_rule_cmd.large_icon = File.join(Constants::ICONS_DIR, "delete_rule.PNG")
	toolbar.add_item delete_rule_cmd
	
	#Copy Rule command
	copy_rule_cmd = UI::Command.new("copy_rule"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active
		
		prompts = ["Rule to copy: "]
		default = ["1"]
		
		rule_list = ShadeUtils.create_rule_list()
		list = [rule_list]
		input = UI.inputbox prompts, default, list, "Rule to copy"
		if input
			n_rule = input[0].to_i-1
			if n_rule < execution.grammar.rules.size
				rule = execution.grammar.rules[n_rule]
				size = execution.grammar.rules.size
				
				# Obtain shape transformations
				t_left = rule.left.shape_transformation
				t_right = rule.right.shape_transformation
				
				# Obtain the new shapes
				left = rule.left.clone
				right = rule.right.clone
				
				#Paint the rule
				last_id = execution.grammar.rules[size-1].rule_id

				new_rule = ShadeUtils.paint_rule(last_id+1, left, right)
  
				#Transform the shapes
				left.shape_transformation = t_left
				right.shape_transformation = t_right
  
				execution.grammar.add_rule(new_rule)
				project.refresh
			else
				UI.messagebox("Rule " + (n_rule+1).to_s + " doesn't exist")
			end
		end
	}
	copy_rule_cmd.tooltip = "Copy rule"
	copy_rule_cmd.small_icon = File.join(Constants::ICONS_DIR, "copy_rule.PNG")
	copy_rule_cmd.large_icon = File.join(Constants::ICONS_DIR, "copy_rule.PNG")
	toolbar.add_item copy_rule_cmd
	
	# First-Rule Axiom command
	first_rule_axiom_cmd = UI::Command.new("first_rule_axiom"){
		execution = Shade.project.execution
		execution.file_axiom = false
		rule = execution.grammar.rules[0]
		# Add new axiom			
		new_axiom = LabelledShape.new(Array.new, Array.new)
		Sketchup.active_model.layers.each { |layer|
			new_axiom.p[layer.name] = rule.left.p[layer.name].clone
			new_axiom.s[layer.name] = rule.left.s[layer.name].clone
		}
		execution.grammar.axiom = new_axiom
		
		execution.reset
	}
	first_rule_axiom_cmd.tooltip = "Set the axiom to the first-rule-left shape"
	first_rule_axiom_cmd.small_icon = File.join(Constants::ICONS_DIR, "first_rule_axiom.PNG")
	first_rule_axiom_cmd.large_icon = File.join(Constants::ICONS_DIR, "first_rule_axiom.PNG")
	toolbar.add_item first_rule_axiom_cmd
	
	toolbar.add_separator()
	
	# Copy shape command
	copy_shape_cmd = UI::Command.new("copy_shape"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active
		
		prompts = ["Origin Rule ID  ", "Origin Rule part  ", "Destiny Rule ID  ", "Destiny rule part "]
		default = ["1", Constants::LEFT, "1", Constants::RIGHT]
		rule_list = ShadeUtils.create_rule_list()
		list = [rule_list, "#{Constants::LEFT}|#{Constants::RIGHT}", rule_list, "#{Constants::LEFT}|#{Constants::RIGHT}"]
		input = UI.inputbox prompts, default, list, "Copy shape:"
		if input
			origin_rule_idx = input[0].to_i - 1
			origin_rule_part = input[1]
			destiny_rule_idx = input[2].to_i - 1
			destiny_rule_part = input[3]
			ShadeUtils.copy_shape(origin_rule_idx, origin_rule_part, destiny_rule_idx, destiny_rule_part)
			project.refresh
		end
	}
	copy_shape_cmd.tooltip = "Copy shape"
	copy_shape_cmd.small_icon = File.join(Constants::ICONS_DIR, "copy_shape.PNG")
	copy_shape_cmd.large_icon = File.join(Constants::ICONS_DIR, "copy_shape.PNG")
	toolbar.add_item copy_shape_cmd
  
	# Add shape box command
	add_shape_box_command = UI::Command.new("add_shape_box"){
		Sketchup.active_model.close_active
		prompts = ["Rule ID ", "Rule part "]
		default = ["1", "Left"]
		rule_list = ShadeUtils.create_rule_list()
		list = [rule_list, "Left|Right"]
		input = UI.inputbox prompts, default, list, "Choose shape:"
		
		if input
			rule_idx = input[0].to_i - 1
			# Transform the layout transformation of the shapes
			t = Geom::Transformation.new
			rule_idx.times do
				t = Constants::DESCEND_T * t
			end

			rule = Shade.project.execution.grammar.rules[rule_idx]
			if input[1] == "Left"
				t = Constants::LEFT_T * t
				rule.alpha.erase
				rule.alpha = ShadeUtils.create_default_left_shape    
				rule.alpha.layout_transformation =  t
				rule.alpha.paint
				if ((rule_idx == 0) and (!Shade.project.execution.file_axiom))
					new_axiom = ShadeUtils.create_default_axiom
					Shade.project.execution.grammar.axiom = new_axiom
					Shade.project.execution.reset
				end
			else
				t = Constants::RIGHT_T * t
				rule.beta.erase
				rule.beta= ShadeUtils.create_default_right_shape    
				rule.beta.layout_transformation =  t
				rule.beta.paint
			end
		end
	}
	add_shape_box_command.tooltip = "Add Shape Box"
	add_shape_box_command.small_icon = File.join(Constants::ICONS_DIR, "create_shape_box.PNG")
	add_shape_box_command.large_icon = File.join(Constants::ICONS_DIR, "create_shape_box.PNG")
	toolbar.add_item add_shape_box_command
	
	toolbar.add_separator()
	
	
	#Add label command
	add_label_cmd = UI::Command.new("add_label"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active

		prompts = ["Rule ID ", "Rule part ", "Color "]
		default = ["1", Constants::LEFT, "Red"]
		rule_list = ShadeUtils.create_rule_list()
		list = [rule_list, "#{Constants::LEFT}|#{Constants::RIGHT}", "Red|Green|Blue|Yellow|White|Black"]
		input = UI.inputbox prompts, default, list, "Add label to shape:"
		if input
			rule_idx = input[0].to_i - 1
			rule = execution.grammar.rules[rule_idx]
			if input[1] == Constants::LEFT
				shape = rule.left
			else
				shape = rule.right
			end
			color = input[2]
			add_label_tool = AddLabelTool.new(shape, color)
			Sketchup.active_model.select_tool add_label_tool
		end
	}
	add_label_cmd.tooltip = "Add label to shape"
	add_label_cmd.small_icon = File.join(Constants::ICONS_DIR, "add_label.PNG")
	add_label_cmd.large_icon = File.join(Constants::ICONS_DIR, "add_label.PNG")
	toolbar.add_item add_label_cmd
	
	#Show labels command
	show_cmd = UI::Command.new("show_labels"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active
		execution.show_labels = true
		project.refresh()
	}
	show_cmd.tooltip = "Show labels"
	show_cmd.small_icon = File.join(Constants::ICONS_DIR, "show_labels.PNG")
	show_cmd.large_icon = File.join(Constants::ICONS_DIR, "show_labels.PNG")
	toolbar.add_item show_cmd
	
	#Hide labels command
	hide_cmd = UI::Command.new("hide_labels"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active		
		execution.show_labels = false
		project.refresh()
	}
	hide_cmd.tooltip = "Hide labels"
	hide_cmd.small_icon = File.join(Constants::ICONS_DIR, "hide_labels.PNG")
	hide_cmd.large_icon = File.join(Constants::ICONS_DIR, "hide_labels.PNG")
	toolbar.add_item hide_cmd
	
	#Change size of labels command
	change_label_radius_cmd = UI::Command.new("change_label_radius"){
		prompts = ["New radius (between 0.1 and 0.9)"]
		default = [0.5]
		list = [""]
		input = UI.inputbox prompts, default, list, "New radius of labels"
		if input
			Shade.label_radius = input[0].to_f.m
			project = Shade.project
			execution = Shade.project.execution
			Sketchup.active_model.close_active		
			project.refresh(true)
		end
	}
	change_label_radius_cmd.tooltip = "Change label radius"
	change_label_radius_cmd.small_icon = File.join(Constants::ICONS_DIR, "change_label_radius.PNG")
	change_label_radius_cmd.large_icon = File.join(Constants::ICONS_DIR, "change_label_radius.PNG")
	toolbar.add_item change_label_radius_cmd
	
	toolbar.add_separator()
	
	#  command
	convert_2D_cmd = UI::Command.new("convert2D"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active
		
		currentShape = ShadeUtils.current_shape
		currentShape.paint3d = false
		
		
		currentShape = ShadeUtils.current_shape
		if currentShape.dimension
			currentShape.dimension = false
			currentShape.paint3d = false
			currentShape.refresh(true)
		end
		}
		convert_2D_cmd.tooltip = "Convert into 2D shape"
		convert_2D_cmd.small_icon = File.join(Constants::ICONS_DIR, "convert_2D.PNG")
		convert_2D_cmd.large_icon = File.join(Constants::ICONS_DIR, "convert_2D.PNG")
		toolbar.add_item convert_2D_cmd
	
	#  command 3D
	convert_3D_cmd = UI::Command.new("convert3D"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active
		
		
		currentShape = ShadeUtils.current_shape
		currentShape.paint3d = true
		
		cont_layer = 0
		z_values = false
		currentShape.s.each_key{ |layer_name|
			cont_layer += 1
			if (!currentShape.z[layer_name].nil?)
				if ((!currentShape.z[layer_name][0].nil?)||(!currentShape.z[layer_name][1].nil?))
					if ((currentShape.z[layer_name][0] != 0)||(currentShape.z[layer_name][1] != 0))
						z_values = true
					end
				end
			end
		}
		#if we just have z values
		if z_values
			currentShape.refresh(true)
		#Teste if we have a 3D shape
		elsif !currentShape.dimension
			prompts = ["3D options: "]
			default = ["Introduce the 3D values "]
			list = ["Introduce the 3D values |Open the 3D values from a file"]
			input = UI.inputbox prompts, default, list, "3D options"
			if input
				if input.include? "Open the 3D values from a file"
					opened = false
					while !opened
						chosen_3d_path = UI.openpanel("Open 3d values", "" ,"*.txt")
						if chosen_3d_path	
							if ShadeUtils.get_extension(chosen_3d_path) == "txt"				
								begin
									File.open(chosen_3d_path, 'r') do |f|
										line = f.gets
										line_a = line.split
										z_length = line_a.length
										#The file has the same number of layers
										if line_a.length == cont_layer*2
											new_z = Array.new
											for i in 0..line_a.length-1
												if /^[\d]+(\.[\d]+){0,1}$/ === line_a[i]
													new_z[i] = line_a[i]
												else
													raise LoadError, "The file must only contain numbers"
												end
											end
											cont = 0
											z_hash = Hash.new(nil)
											currentShape.s.each_key { |layer_name|
												z_hash[layer_name] = Array.new(2)
												if cont < z_length
													z_hash[layer_name][0] = new_z[cont]
													cont += 1
													z_hash[layer_name][1] = new_z[cont]
												elsif
													z_hash[layer_name][0] = 0
													z_hash[layer_name][1] = 0
												end
												cont += 1
											}
											ShadeUtils.convert3d(z_hash)

										else
											UI.messagebox("The path you have chosen is not valid (it must be two numbers per layer, #{cont_layer*2})")
										end
									end
									opened = true
								rescue
									UI.messagebox("The path you have chosen is not valid (maybe it has some special character?)")
								rescue LoadError => e
									UI.messagebox(e.message)
								end
							else
								UI.messagebox("Please choose a .txt file")
							end
						else
							opened = true
						end
					end
				elsif input.include? "Introduce the 3D values "
					# Find and show our html file
					html_path = Sketchup.find_support_file "writezlayers.html" ,Constants::HTML_DIR
					z_dialog = write_z_layers("Z")
					z_dialog.set_file(html_path)
					z_dialog.show()
				end
			end
		end
	}
	convert_3D_cmd.tooltip = "Convert into 3D shape"
	convert_3D_cmd.small_icon = File.join(Constants::ICONS_DIR, "convert_3D.PNG")
	convert_3D_cmd.large_icon = File.join(Constants::ICONS_DIR, "convert_3D.PNG")
	toolbar.add_item convert_3D_cmd
	
	toolbar.show()
end

#Method that creates the toolbar for performing execution tasks 
def create_execution_toolbar()
 
	toolbar = UI.toolbar Constants::EXECUTION_TOOLBAR_NAME
  
	# Execute command
	exe_cmd = UI::Command.new("apply_rule"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active

		prompts = ["Chosen rule id"]
		default = [1]
		rule_list = ShadeUtils.create_rule_list()
		list = [rule_list]
		input = UI.inputbox prompts, default, list, "Apply Rule"
		if input
			chosen_rule_idx = input[0]
			chosen_rule_id = execution.grammar.rules[chosen_rule_idx.to_i - 1].rule_id
			
			#STEP 1: Save temp files
			save_temp_files()

			#STEP 2: Call external command with argument: chosen_rule_id
			command_directory = "#{File.dirname(__FILE__)}/#{Constants::COMMANDS_DIR}/execute-command.rb"
			command_directory.gsub!("/", "\\")
			output = system("ruby \"#{command_directory}\" #{chosen_rule_id}")
			
			#STEP 3: Catch return from external command
			if !($? == 0)
				UI.messagebox("The loaded constraints/goals use some functions of SketchUp API. The execution will be performed inside the SketchUp environment, and it may take more time.")
				#puts output
				success = execution.apply_rule(chosen_rule_id)
				if !success
					UI.messagebox("The rule cannot be applied")
				end
			else
				log_directory = "#{File.dirname(__FILE__)}/#{Constants::TEMP_DIR}/result.log"
				log_directory.gsub!("/", "\\")
				File.open(log_directory, 'r') do |f|
					line = f.gets.strip
					if (line == "true")
						load_temp_files()
					elsif (line == "false")
						UI.messagebox("The rule cannot be applied")
					end
				end	
			end
			
			#STEP 4: delete files of temporal directory
			delete_temp_files()
		end
	}
	exe_cmd.tooltip = "Apply rule"
	exe_cmd.small_icon = File.join(Constants::ICONS_DIR, "execute.PNG")
	exe_cmd.large_icon = File.join(Constants::ICONS_DIR, "execute.PNG")
	toolbar.add_item exe_cmd
	
	# Ramdom Execute command
	rand_exe_cmd = UI::Command.new("apply_random_rule"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active
		
		#STEP 1: Save temp files
		save_temp_files()
		
		#STEP 2: Call external command with argument: chosen_rule_id
		command_directory = "#{File.dirname(__FILE__)}/#{Constants::COMMANDS_DIR}/execute-random-command.rb"
		command_directory.gsub!("/", "\\")
		system("ruby \"#{command_directory}\"")

		#STEP 3: Catch return from external command
		if !($? == 0)
			UI.messagebox("The loaded constraints/goals use some functions of SketchUp API. The execution will be performed inside the SketchUp environment, and it may take more time.")
			success = execution.apply_rule_random()
			if !success
				UI.messagebox("No rule can be applied")
			end
		else
			log_directory = "#{File.dirname(__FILE__)}/#{Constants::TEMP_DIR}/result.log"
			log_directory.gsub!("/", "\\")
			File.open(log_directory, 'r') do |f|
				line = f.gets.strip
				if (line == "true")
					load_temp_files()
				elsif (line == "false")
					UI.messagebox("No rule can be applied")
				end
			end	
		end
		
		#STEP 4: delete files of temporal directory
		delete_temp_files()
	}
	rand_exe_cmd.tooltip = "Apply random rule"
	rand_exe_cmd.small_icon = File.join(Constants::ICONS_DIR, "execute_random.PNG")
	rand_exe_cmd.large_icon = File.join(Constants::ICONS_DIR, "execute_random.PNG")
	toolbar.add_item rand_exe_cmd
	
	# Random n-execute command
	rand_n_exe_cmd = UI::Command.new("apply_n_random_rules"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active
		
		prompts = ["Number of rules", "See steps?"]
		default = ["0", "No"]
		list = ["", "No|Yes"]
		input = UI.inputbox prompts, default, list, "Apply n random rules"
		if input
			show_backtracking_steps = false
			if input[1] == "Yes"
				show_backtracking_steps = true
			end
			if show_backtracking_steps
				n_applied = execution.apply_n_rules_random(input[0].to_i, show_backtracking_steps)
				if n_applied
					UI.messagebox("Success. Number of steps: "+execution.backtracking_steps.to_s)
				else
					UI.messagebox("Failure. Impossible to apply " + input[0] + " rules.")
				end
			else #Try external execution
				#STEP 1: Save temp files
				save_temp_files()
		
				#STEP 2: Call external command with argument: chosen_rule_id
				command_directory = "#{File.dirname(__FILE__)}/#{Constants::COMMANDS_DIR}/n-execute-random-command.rb"
				command_directory.gsub!("/", "\\")
				system("ruby \"#{command_directory}\" #{input[0].to_i}")
				
				#STEP 3: Catch return from external command				
				if !($? == 0)
					UI.messagebox("The loaded constraints/goals use some functions of SketchUp API. The execution will be performed inside the SketchUp environment, and it may take more time.")
					success = execution.apply_n_rules_random(input[0].to_i, false)
					if !success
						UI.messagebox("The rule cannot be applied")
					end
				else
					log_directory = "#{File.dirname(__FILE__)}/#{Constants::TEMP_DIR}/result.log"
					log_directory.gsub!("/", "\\")
					File.open(log_directory, 'r') do |f|
						line = f.gets.strip
						if (line == "false")
							UI.messagebox("Failure. Impossible to apply #{input[0]} rules.")
						else
							load_temp_files()
							UI.messagebox("Success. Number of steps: #{line}")
						end
					end	
				end
				
				#STEP 4: delete files of temporal directory
				delete_temp_files()
			end
		end
	}
	rand_n_exe_cmd.tooltip = "Apply n random rules"
	rand_n_exe_cmd.small_icon = File.join(Constants::ICONS_DIR, "execute_n_random.PNG")
	rand_n_exe_cmd.large_icon = File.join(Constants::ICONS_DIR, "execute_n_random.PNG")
	toolbar.add_item rand_n_exe_cmd
	
	# Random goal-execute command
	rand_goal_exe_cmd = UI::Command.new("apply_goal_random_rules"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active
		
		if !execution.goals.empty?
		
			prompts = ["See steps?", "Timeout (secs)?", "Maximum number of rules?"]
			default = ["No", "No timeout", "100"]
			list = ["No|Yes", "15|30|60|No timeout", ""]
			input = UI.inputbox prompts, default, list, "Apply random rules until goals are achieved"
			
			if input
				show_backtracking_steps = false
				if input[0] == "Yes"
					show_backtracking_steps = true
				 end
				 timeout = 0
				if input[1] != "No timeout"
					timeout = input[2].to_i
				end
				if input[2] != "No maximum"
					n_max_rules = input[2].to_i
				end

				if show_backtracking_steps
					t1 = Time.now
					n_applied = execution.apply_goal_rules_random(show_backtracking_steps, timeout, n_max_rules)
					t2 = Time.now
					if n_applied
						UI.messagebox("Success. Number of steps: "+execution.backtracking_steps.to_s)
					else
						UI.messagebox("Failure. Impossible to comply the goals")
					end
					puts "Elapsed time: #{t2 - t1}"
				else
					#STEP 1: Save temp files
					save_temp_files()
          
					#STEP 2: Call external command with argument: chosen_rule_id
					command_directory = "#{File.dirname(__FILE__)}/#{Constants::COMMANDS_DIR}/random-goal-execute-command.rb"
					command_directory.gsub!("/", "\\")
					t1 = Time.now
					system("ruby \"#{command_directory}\" #{timeout} #{n_max_rules}")
					t2 = Time.now
					
					#STEP 3: Catch return from external command				
					if !($? == 0)
						UI.messagebox("The loaded constraints/goals use some functions of SketchUp API. The execution will be performed inside the SketchUp environment, and it may take more time.")
						t1 = Time.now
						n_applied = execution.apply_goal_rules_random(show_backtracking_steps, timeout, n_max_rules)
						t2 = Time.now
						puts "Elapsed time: #{t2 - t1}"
						if n_applied == -1
							UI.messagebox("No goals are speficied")
						elsif n_applied
							UI.messagebox("Success. Number of steps: #{execution.backtracking_steps.to_s}")
						else
							UI.messagebox("Failure. Impossible to satisfy the specified goals")
						end
					else
						log_directory = "#{File.dirname(__FILE__)}/#{Constants::TEMP_DIR}/result.log"
						log_directory.gsub!("/", "\\")
						File.open(log_directory, 'r') do |f|
							line = f.gets
							if (line == "false")
								UI.messagebox("Failure. Impossible to satisfy the specified goals")
							else
								load_temp_files()
								UI.messagebox("Success. Number of steps: #{line}")
								puts "Elapsed time: #{t2 - t1}"
							end
						end	
					end
					
					#STEP 4: delete files of temporal directory
					delete_temp_files()
				end
			end
		else
			UI.messagebox("No goals are speficied")
		end
		
	}
	rand_goal_exe_cmd.tooltip = "Apply random rules until goals are achieved"
	rand_goal_exe_cmd.small_icon = File.join(Constants::ICONS_DIR, "execute_goal_random.PNG")
	rand_goal_exe_cmd.large_icon = File.join(Constants::ICONS_DIR, "execute_goal_random.PNG")
	toolbar.add_item rand_goal_exe_cmd
	
	# Execute robot command
	execute_robot_command = UI::Command.new("execute_script"){
		Sketchup.active_model.close_active
		if Shade.robot
			prompts = ["Number of designs"]
			default = ["1"]
			list = []
			input = UI.inputbox prompts, default, list, "Number of designs to generate"
			
			if input
				chosen_design_path = UI.savepanel("Save designs", "" ,"design.txt")
				if chosen_design_path			
					
					Shade.robot.execute(input[0], File.dirname(chosen_design_path))
					Shade.project.refresh(true)
					UI.beep
				end
			end
		else
			UI.messagebox("No script has been loaded")
		end
	}
	execute_robot_command.tooltip = "Execute script"
	execute_robot_command.small_icon = File.join(Constants::ICONS_DIR, "execute_robot.PNG")
	execute_robot_command.large_icon = File.join(Constants::ICONS_DIR, "execute_robot.PNG")
	toolbar.add_item execute_robot_command
		
	toolbar.add_separator()
	
	# Undo step command
	undo_step_cmd = UI::Command.new("undo_step"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active
		
		execution.undo
	}
	undo_step_cmd.tooltip = "Undo Step"
	undo_step_cmd.small_icon = File.join(Constants::ICONS_DIR, "undo.PNG")
	undo_step_cmd.large_icon = File.join(Constants::ICONS_DIR, "undo.PNG")
	toolbar.add_item undo_step_cmd
	
	# Reset command
	reset_cmd = UI::Command.new("reset"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active
		
		execution.reset
	}
	reset_cmd.tooltip = "Reset"
	reset_cmd.small_icon = File.join(Constants::ICONS_DIR, "reset.PNG")
	reset_cmd.large_icon = File.join(Constants::ICONS_DIR, "reset.PNG")
	toolbar.add_item reset_cmd

	toolbar.show
end

#Method that creates the toolbar for dealing with constraints and goals
def create_guidance_toolbar
	toolbar = UI.toolbar Constants::GUIDANCE_TOOLBAR_NAME

	#Add constraint command
	add_constraint_cmd = UI::Command.new("add_constraint"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active
		
		prompts = ["Constraint: "]
		default = [Constants::DISTINCT_SHAPE_CONSTRAINT_NAME]
		
		array_list = ""
		Shade.constraint_class_names.each { |pair|
			array_list += "#{pair[1]}|"
		}
		list = [array_list[0..array_list.size-2]]
		input = UI.inputbox prompts, default, list, "Constraint to add"
		if input
			result = false
			if input[0] == Constants::AREA_CONSTRAINT_NAME
				prompts = ["Points: "]
				default = ["Default"]
				list = ["Default|From File"]
				input = UI.inputbox prompts, default, list, "Choose points"
				if input
					if input[0] == "Default"
						points = Constants::PTS_AREA.clone
					elsif input[0] == "From File"
						# TODO: make compatible with different OOSS (masks)
						chosen_area_path = UI.openpanel("Open area", "" ,"*.area")	
						points = []
						File.open(chosen_area_path, 'r') do |f|
							while line = f.gets
								line_a = line.split
								point = Array.new
								point[0] = line_a[0].to_f.m
								point[1] = line_a[1].to_f.m
								point[2] = line_a[2].to_f.m
								points.push point
							end
						end
					end
					result = project.execution.add_constraint(AreaConstraint.new(points))
				end
			else
				i = 0
				found = false
				constraint_class = nil
				while ((i < Shade.constraint_class_names.size) && !found)
					if (Shade.constraint_class_names[i][1] == input[0])
						found = true
						constraint_class = Shade.constraint_class_names[i][0]
					end
					i+=1
				end
				if constraint_class
					result = project.execution.add_constraint(constraint_class.new)
				end
			end
			if !result
				UI.messagebox("The grammar already has the "+input[0]+" Constraint")
			end
		end
	}
	add_constraint_cmd.tooltip = "Add Constraint"
	add_constraint_cmd.small_icon = File.join(Constants::ICONS_DIR, "add_constraint.PNG")
	add_constraint_cmd.large_icon = File.join(Constants::ICONS_DIR, "add_constraint.PNG")
	toolbar.add_item add_constraint_cmd
	
	#Add custom constraint command
	add_custom_constraint_cmd = UI::Command.new("add_custom_constraint"){

		# Find and show our html file
		html_path = Sketchup.find_support_file "chooseName.html" ,Constants::HTML_DIR
		name_dialog = create_name_dialog("Constraint")
		name_dialog.set_file(html_path)
		name_dialog.show()
	}
	add_custom_constraint_cmd.tooltip = "Add Custom Constraint"
	add_custom_constraint_cmd.small_icon = File.join(Constants::ICONS_DIR, "add_custom_constraint.PNG")
	add_custom_constraint_cmd.large_icon = File.join(Constants::ICONS_DIR, "add_custom_constraint.PNG")
	toolbar.add_item add_custom_constraint_cmd
	
	#Remove constraint command
	remove_constraint_cmd = UI::Command.new("remove_constraint"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active

		prompts = ["Constraint: "]
		default = [Constants::DISTINCT_SHAPE_CONSTRAINT_NAME]
		
		array_list = ""
		Shade.constraint_class_names.each { |pair|
			array_list += "#{pair[1]}|"
		}
		list = [array_list[0..array_list.size-2]]
		input = UI.inputbox prompts, default, list, "Constraint to remove"
		if input
			result = project.execution.delete_constraint(input[0])
			if !result
				UI.messagebox("The grammar does not have the "+input[0]+" Constraint")
			end
		end
		#Here we do not have to reset the design, as the execution will be the same
	}
	remove_constraint_cmd.tooltip = "Remove Constraint"
	remove_constraint_cmd.small_icon = File.join(Constants::ICONS_DIR, "remove_constraint.PNG")
	remove_constraint_cmd.large_icon = File.join(Constants::ICONS_DIR, "remove_constraint.PNG")
	toolbar.add_item remove_constraint_cmd
	
	# Project Constraints command
	project_constraints_cmd = UI::Command.new("project_constraints"){
		project = Shade.project
		execution = Shade.project.execution
		constraints = ""
		execution.constraints.each{ |r|
			constraints = constraints + r.name + "\n"
		}
		UI.messagebox("Present Constraints on #{project.title}:\n "+constraints)
	}
	project_constraints_cmd.tooltip = "Project constraints"
	project_constraints_cmd.small_icon = File.join(Constants::ICONS_DIR, "project_constraints.PNG")
	project_constraints_cmd.large_icon = File.join(Constants::ICONS_DIR, "project_constraints.PNG")
	toolbar.add_item project_constraints_cmd
	
	toolbar.add_separator
		
	# Add goal command
	add_goal_command = UI::Command.new("add_goal"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active
		
		prompts = ["Goal: "]
		default = [Constants::NO_LABELS_GOAL_NAME]
		
		array_list = ""
		Shade.goal_class_names.each { |pair|
			array_list += "#{pair[1]}|"
		}
		list = [array_list[0..array_list.size-2]]
		input = UI.inputbox prompts, default, list, "Goal to add"
		if input
			result = false
			i = 0
			found = false
			goal_class = nil
			while ((i < Shade.goal_class_names.size) && !found)
				if (Shade.goal_class_names[i][1] == input[0])
					found = true
					goal_class = Shade.goal_class_names[i][0]
				end
				i+=1
			end
			if goal_class
				result = project.execution.add_goal(goal_class.new)
			end
			if !result
				UI.messagebox("The grammar already has the "+input[0]+" goal")
			end
		end
	}
	add_goal_command.tooltip = "Add Goal"
	add_goal_command.small_icon = File.join(Constants::ICONS_DIR, "add_goal.PNG")
	add_goal_command.large_icon = File.join(Constants::ICONS_DIR, "add_goal.PNG")
	toolbar.add_item add_goal_command
	
	#Add custom goal command
	add_custom_goal_cmd = UI::Command.new("add_custom_goal"){
		# Find and show our html file
		html_path = Sketchup.find_support_file "chooseName.html" ,Constants::HTML_DIR
		name_dialog = create_name_dialog("Goal")
		name_dialog.set_file(html_path)
		name_dialog.show()
	}
	add_custom_goal_cmd.tooltip = "Add Custom Goal"
	add_custom_goal_cmd.small_icon = File.join(Constants::ICONS_DIR, "add_custom_goal.PNG")
	add_custom_goal_cmd.large_icon = File.join(Constants::ICONS_DIR, "add_custom_goal.PNG")
	toolbar.add_item add_custom_goal_cmd
	
	# Remove goal command
	remove_goal_command = UI::Command.new("remove_goal"){
		project = Shade.project
		execution = Shade.project.execution
		Sketchup.active_model.close_active

		prompts = ["Goal: "]
		default = [Constants::NO_LABELS_GOAL_NAME]
		
		array_list = ""
		Shade.goal_class_names.each { |pair|
			array_list += "#{pair[1]}|"
		}
		list = [array_list[0..array_list.size-2]]
		input = UI.inputbox prompts, default, list, "Goal to remove"
		if input
			result = project.execution.delete_goal(input[0])
			if !result
				UI.messagebox("The grammar does not have the "+input[0]+" goal")
			end
		end
	}
	remove_goal_command.tooltip = "Remove Goal"
	remove_goal_command.small_icon = File.join(Constants::ICONS_DIR, "remove_goal.PNG")
	remove_goal_command.large_icon = File.join(Constants::ICONS_DIR, "remove_goal.PNG")
	toolbar.add_item remove_goal_command
	
	# Project Goals command
	project_goals_cmd = UI::Command.new("project_goals"){
		project = Shade.project
		execution = Shade.project.execution
		goals = ""
		execution.goals.each{ |g|
			goals = goals + g.name + "\n"
		}
		UI.messagebox("Present Goals on #{project.title}:\n "+goals)
	}
	project_goals_cmd.tooltip = "Project Goals"
	project_goals_cmd.small_icon = File.join(Constants::ICONS_DIR, "project_goals.PNG")
	project_goals_cmd.large_icon = File.join(Constants::ICONS_DIR, "project_goals.PNG")
	toolbar.add_item project_goals_cmd
	
	# Satisfy Goals command
	satisfy_goals_command = UI::Command.new("satisfy_goals"){
		project = Shade.project
		execution = Shade.project.execution
		if execution.goals_satisfied?
			UI.messagebox("The goals are satisfied")
		else
			UI.messagebox("The goals are not satisfied")
		end
	}
	satisfy_goals_command.tooltip = "Satisfy Goals?"
	satisfy_goals_command.small_icon = File.join(Constants::ICONS_DIR, "satisfy_goals.PNG")
	satisfy_goals_command.large_icon = File.join(Constants::ICONS_DIR, "satisfy_goals.PNG")
	toolbar.add_item satisfy_goals_command

	toolbar.show

end